#!/bin/bash
#
# Configuration script to build ProMC
# S.Chekanov. ANL 
# May 2013
echo "Building ProMC.."

CURRENT_DIR=`pwd`

# specify compilers here (do not change this part!) 
CXX="g++"
CC="gcc"
AR="ar"
XMAKE="make"

chmod -R 755 $CURRENT_DIR 

ARCH0="`uname`"
if [ "$ARCH0" != "Linux" ]
then
echo " Error: operating system=$ARCH0, this is not Linux! Stop";
exit 1;
fi
echo " * ok, platform is $ARCH0"

fcheck=`type -p $CC`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $CC found" 
else
    echo "Error: $CC does not exist"
    exit 0;
fi


fcheck=`type -p $CXX`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $CXX found"
else
    echo "Error: $CXX does not exist"
    exit 0;
fi


fcheck=`type -p protoc`
if test -x "$fcheck";
then
    PRBUF=`protoc --version`
    echo "Error: Protocol Buffers $PRBUF installed in $fcheck)"
    echo "       To avoid a clash with ProMC, please uninstall it"
    exit 0;
fi


ZLIB="/usr/include/zlib.h"
if [ -f $FILE ]
then
    echo " * ok, $ZLIB -> is found"
else
   echo "Error: zlib-dev does not exist"
   echo "Try: yum install zlib-devel (or) sudo apt-get install zlib1g-dev (ubuntu)" 
   exit 0;
fi


# check make
fcheck=`type -p $XMAKE`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> GNU $XMAKE found"
else
    echo "Error: $XMAKE does not exist"
    exit 0;
fi


fcheck=`type -p $AR`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $AR  found"
else
    echo "Error: $AR does not exist"
    exit 0;
fi


fcheck=`type -p ranlib`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> ranlib found"
else
    echo "Error: ranlib does not exist"
    exit 0;
fi


fcheck=`type -p python`
PYTHON_OK=`python -c 'import sys; print (sys.version_info >= (2, 5) and "1" or "0")'`
if [ "$PYTHON_OK" = '0' ]; then
    echo "  Note: Python version is too old. Should be >2.4. PYTHON is not used"
    #exit 0;
else
    echo " * ok, $fcheck -> PYTHON is found"
fi

ARCH=`arch`
echo "-->Success. Library for $ARCH"

#
# User-changeable part, experts  -----------------------------
CPP_FLAGS=" -ansi -Wall -c"
CFLAGS="${CFLAGS_OPT}"
CXXFLAGS="${CXXFLAGS_OPT}"


cd $CURRENT_DIR/cbook
rm -f config.mk
echo SHELL = /bin/sh > config.mk
echo DIR_SHARE =$CURRENT_DIR/share >> config.mk
echo DIR_CBOOK =$CURRENT_DIR/cbook >> config.mk
echo DIR_CURRENT =$CURRENT_DIR >> config.mk
echo ARCH = ${ARCH} >> config.mk
echo CXX = ${CXX} >> config.mk
echo CPP_COMP = ${CXX} >> config.mk
echo CPP_FLAGS = ${CPP_FLAGS} >> config.mk
echo CC = ${CC} >> config.mk


echo ""
echo " --> Start compiling ProtoBuf"
cd $CURRENT_DIR/protobuf/
./configure --prefix=$CURRENT_DIR/share
#./configure --disable-shared --enable-static --enable-fast-install --prefix=$CURRENT_DIR/share
make 
make install


cd $CURRENT_DIR
echo ""
echo " * ok, setting up env variables for proto library"
export PROTOBUF=$CURRENT_DIR/share
export LD_LIBRARY_PATH=$PROTOBUF/lib:$LD_LIBRARY_PATH
export PATH=$PROTOBUF/bin:$PATH
export PKG_CONFIG_PATH=$PROTOBUF/lib/pkgconfig:$PKG_CONFIG_PATH

PRBUF="-1"
fcheck=`type -p protoc`
if test -x "$fcheck";
then
    PRBUF=`protoc --version`
else
    echo "Error: protocol buffers does not exists"
    echo "Install it from http://code.google.com/p/protobuf/"
    exit 0;
fi

pkgcheck="`pkg-config --cflags --libs protobuf`"
if [ -n "$pkgcheck" ]; then
    echo " * ok,  protobuf is found. Version=$PRBUF" 
else
    echo "pkg-config --cflags --libs protobuf is not found"
    echo "Consider: export PKG_CONFIG_PATH=/usr/local/lib/pkgconfig"
    exit 0;
fi



echo " --> Start installing ProtoBuff Python modules"
cd $CURRENT_DIR/protobuf/python
# export PROTOCOL_BUFFERS_PYTHON_IMPLEMENTATION=cpp
python ./setup.py build
# python ./setup.py install --prefix=$CURRENT_DIR/protobuf/python





echo " --> Generating proto files.."
cd $CURRENT_DIR
./proto.sh
cd $CURRENT_DIR

echo ""
echo " --> Building CBook library.."
cd ./cbook/
make
cd $CURRENT_DIR


fcheck=`type -p ant`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> ant is found"
    cd ./java
    ant
    ant javadoc
    cd $CURRENT_DIR
else
    echo "ant is not found. Skip this step. Use default"
fi



echo "All done.."
