/* ---------------------------------------------------------------------------
** This software is in the public domain, furnished "as is", without technical
** support, and with no warranty, express or implied, as to its usefulness for
** any purpose.
**
**  A library for HEP events storage and processing based on Google's ProtocolBuffers 
**
** Author: S.Chekanov (ANL). chekanov@anl.gov
** Copyright  March, 2013 
** -------------------------------------------------------------------------*/



#include "ProMCBook.h"

bool replace(std::string& str, const std::string& from, const std::string& to) {
    size_t start_pos = str.find(from);
    if(start_pos == std::string::npos)
        return false;
    str.replace(start_pos, from.length(), to);
    return true;
}



ProMCBook::ProMCBook(){ nev=0; isToWrite=false; }
 
ProMCBook::~ProMCBook(){ }

ProMCBook::ProMCBook(const char* filename, const char* option){

  open(filename,option);

 }


// open for writing
void ProMCBook::open(const char* filename, const char* option) { 

     //cout << "PoMC Version: " << PROMCBOOK_VERSION << endl;


     GOOGLE_PROTOBUF_VERIFY_VERSION;
     nev=0;

     if (strcmp(option,"w")==0) {
       isToWrite=true;
       cout << "ProMCBook: Writing file = " << filename << endl;
       outzip = new ZipOutputStream(filename);
       description="none";

       outzip->putNextEntry( "version" ) ;
       std::stringstream out;
       out << current_version;
       (*outzip) << out.str();


     }  else if (strcmp(option,"r")==0) {
        isToWrite=false;
        cout << "ProMCBook: Reading file = " << filename << endl; 
        inpzip = new ZipInputStream(filename);
    try {

   // version 
   std::stringstream sout;
   sout << inpzip->rdbuf();
   string ss=sout.str();
   version = atoi(ss.c_str());


   next();

   try {

      Description h;
      std::stringstream sout;
      sout << inpzip->rdbuf();
      h.ParseFromIstream(&sout);

      // get info
      version = h.version(); 
      description=h.description();
      timestamp=h.timestamp();
      nev=h.events();
      cout << "ProMCBook: version = " << version << endl;
      cout << "ProMCBook: events = " << nev << endl;
      cout << "ProMCBook: description = " << description << endl;

 
   } catch( IOException &e ) {
    cerr << "IOException caught in fetching description:" << endl ;
    cerr << e.what() << endl ;
   }
   catch( ... ) {
    cerr << "Unspecified exception caught in fetching description:" << endl ;
  }








} catch( IOException &e ) {
    cerr << "IOException caught in ProMCBook:" << endl ;
    cerr << e.what() << endl ;
  }
  catch( ... ) {
    cerr << "Unspecified exception caught in ProMCBook:" << endl ;
  }

 }



/*
  try {
    zipios::ConstEntryPointer entry =inpzip->getNextEntry() ;
    if(entry->isValid()) { 
          std::stringstream sout;
          sout << inpzip->rdbuf();
          header.ParseFromIstream(&sout);
    }
  } catch( IOException &e ) {
    cerr << "IOException caught in ProMCBook:" << endl ;
    cerr << e.what() << endl ;
   }
   catch( ... ) {
      cerr << "Unspecified exception caught in ProMCBook:" << endl ;
  }
*/
  

}    



// get size 
unsigned int  ProMCBook::size()
{
    return  nev;
}


/*
// get header 
ProMCHeader ProMCBook::getHeader()
{
    return  header;
}
*/





/*
  zipios::ConstEntryPointer entry =inpzip->getNextEntry() ;

  if (entry && entry->isValid()) {
   // return entry->toString();
      cout << "Reading " << endl;
  }
*/

//        cout << "Debug reading=" << "main02.nbu" << endl;
//        std::ifstream file("main02.nbu", std::ios::in | std::ios::binary);
//        zipios::ZipInputStream zipstream(file);
//        zipios::ConstEntryPointer entry;
//        entry = zipstream.getNextEntry();
/*
        while (entry->isValid() && entry->getName() != "thumbnails/Thumbnail.png")
            entry = zipstream.getNextEntry();
        if (entry && entry->isValid()) {
            // ok, we have found the file. Now, read it in byte for byte
                    std::istream *str = &zipstream;
                    std::vector<unsigned char> content;
                    unsigned char c;
                    while (str->get((char&)c)) {
                    content.push_back(c);
                            }
          }
       } 
          
 */        
           

// get next record
int  ProMCBook::next(){


  try {
  if (inpzip == NULL) return 1; 
  if (inpzip->eof()) return 2;
  if (inpzip->fail()) return 3;  
  if (inpzip->bad()) return 4; 
  zipios::ConstEntryPointer entry =inpzip->getNextEntry() ;
  if(!entry->isValid()) return 5; 

  } catch( IOException &e ) {
   // cerr << "IOException caught in main:" << endl ;
   // cerr << e.what() << endl ;
   return 6;
  }
  catch( ... ) {
    //cerr << "Unspecified exception caught in main:" << endl ;
    return 7;
  }
 

  return 0;
}



// get header
ProMCHeader ProMCBook::getHeader(){

// go to next event
next();

ProMCHeader h;

try {
   std::stringstream sout;
   sout << inpzip->rdbuf();
   h.ParseFromIstream(&sout);
} catch( IOException &e ) {
    cerr << "IOException caught in main:" << endl ;
    cerr << e.what() << endl ;
  }
  catch( ... ) {
    cerr << "Unspecified exception caught in main:" << endl ;
  }

return h;

}



// get next record
ProMCEvent ProMCBook::get(){

ProMCEvent eve;

try {

   std::stringstream sout;
   sout << inpzip->rdbuf();
   eve.ParseFromIstream(&sout);
   return eve;

//   if (inpzip->good()) 
//                    return eve;

 
} catch( IOException &e ) {
    cerr << "IOException caught in main:" << endl ;
    cerr << e.what() << endl ;
  }
  catch( ... ) {
    cerr << "Unspecified exception caught in main:" << endl ;
  }


/*
   zipios::ConstEntryPointer entry =inpzip->getNextEntry() ;

  if (entry->isValid()) {

    ProMCEvent eve;

    // entry->toString().c_str()
    fstream input;
    input<<entry;
//    std::istream *str = &zipstream;

    eve.ParseFromIstream(&input);
    

   } 
*/

  return eve; 

}



// print all
void ProMCBook::clear()
{

}

// header file
void ProMCBook::setHeader( ProMCHeader h ) {

         std::string out;
         if (!h.SerializeToString(&out)) {
            cerr << "Failed to write header" << endl;
         }
          outzip->putNextEntry( "header" ) ;
          (*outzip) << out;
          h.Clear();
}


// set statistics information after the events 
void ProMCBook::setStatistics( ProMCStat h ) {

         std::string out;
         if (!h.SerializeToString(&out)) {
            cerr << "Failed to write statistics information" << endl;
         }
          outzip->putNextEntry( "statistics" ) ;
          (*outzip) << out;
          h.Clear();
}


// set the description for this file
void ProMCBook::setDescription( int events, string  describe ) {


        description=describe;

        std::time_t t = std::time(0);  // t is an integer type

        Description  eve;
        eve.set_version(current_version);
        eve.set_description(description);
        eve.set_events(events);
        eve.set_timestamp((int)t);
        std::string out;
        if (!eve.SerializeToString(&out)) {
            cerr << "Failed to write description" << endl;
         }
        outzip->putNextEntry( "description" ) ;
        (*outzip) << out;
        eve.Clear();



}





// serialize to string
void ProMCBook::write( ProMCEvent eve ) { 

         std::string out;
         if (!eve.SerializeToString(&out)) {
            cerr << "Failed to write event" << endl;
         }

          std::stringstream sout;
          sout << nev;
          outzip->putNextEntry( sout.str() ) ;
          (*outzip) << out;
          nev++;
          eve.Clear();
}



// write file description and close
void ProMCBook::close() { 


   if (isToWrite==true) {



       cout << " ##### Closing ProMC ##### " << endl; 


       // write log file if exists
       string filename="logfile.txt"; 
       ifstream ifs1( filename.c_str(), ios::in | ios::binary );
       if(!ifs1.fail()) {
           outzip->putNextEntry(filename);
           (*outzip) << ifs1.rdbuf() ;
           ifs1.close();
           cout << " Info: File=" << filename << " is attached" << endl;
       if (outzip->fail() || outzip->bad()) 
           cout << "Problem with writing "<< filename << endl;
       } else {
           cout << " Info: File=" << filename << " not found. No log file is attached" << endl;
       }    



       // write number of processed events
       outzip->putNextEntry( "promc_nevents" ) ;
       std::stringstream out1;
       out1 << nev;
       (*outzip) << out1.str();

        // description
       outzip->putNextEntry( "promc_description" ) ;
       std::stringstream out2;
       out2 << description;
       (*outzip) << out2.str();
        

       bool ierr=false;
       filename="proto/ProMCHeader.proto"; 
       ifstream ifs2( filename.c_str(), ios::in | ios::binary ) ;
       if(!ifs2.fail()) {
           replace(filename, "proto/", "");
           outzip->putNextEntry(filename);
           (*outzip) << ifs2.rdbuf() ;
           ifs2.close();
       if (outzip->fail()) 
           cout << " Problem with writing "<< filename << endl;
           //ierr=true;
       } else {
           cout << " Header file=" << filename << " not found" << endl;
           ierr=true;
       } 

       filename="proto/ProMC.proto";
       ifstream ifs3( filename.c_str(), ios::in | ios::binary ) ;
       if(!ifs3.fail()) {
           replace(filename, "proto/", "");
           outzip->putNextEntry(filename);
           (*outzip) << ifs3.rdbuf() ;
           ifs3.close();
       if (outzip->fail())
           cout << " Problem with writing "<< filename << endl;
           //ierr=true;
       } else {
           cout << " Event record file=" << filename << " not found" << endl;
           ierr=true;
       }


       filename="proto/ProMCStat.proto";
       ifstream ifs4( filename.c_str(), ios::in | ios::binary ) ;
       if(!ifs4.fail()) {
           replace(filename, "proto/", "");
           outzip->putNextEntry(filename);
           (*outzip) << ifs4.rdbuf() ;
           ifs4.close();
       if (outzip->fail())
           cout << " Problem with writing "<< filename << endl;
           //ierr=true;
       } else {
           cout << " Statistics file=" << filename << " not found" << endl;
           ierr=true;
       }

    
       if (ierr){
          cout << " -> Warning: Not self-describing file format." << endl;
          cout << "    To make it self-describing, put *.proto files to the directory proto/" << endl;
         } 

       outzip->close();
       cout << " ##### ProMC file is closed #####" << endl; 

    }

 
 // delete outzip;
 // delete inpzip;
  if (isToWrite==false) inpzip->close();
  clear();
}




